module paint_mod3d_mpi
!!! pass the function calc_cut an image, eigenvector, a cut threshold, and a weight matrix 
!!! and it will return the normalized cut value 
  use lint2
  use sep
  use warp_mod
  use sorting_mod
  use mpi_sep
  logical, private , save :: first
   integer,private,save :: nlocal,impi,nmpi
  character(len=128), private,save :: local_i,local_v

contains

  subroutine sparse_mat_init()
     first=.true.
  end subroutine

  subroutine distribute_mat(ntot)
    integer ::ndone,nblock,ntot
    integer :: i,ibeg,iend
    integer :: nsmall,nextra
    real,pointer  :: buf(:,:)
    integer status(MPI_STATUS_SIZE) ,ierr
    integer, pointer :: iloc(:,:)
    real, pointer    :: val(:) 
    
   

    impi=sep_thread_num()
    nmpi=sep_num_thread()
    ndone=0
    nlocal=0
    allocate(buf(3,10000))
    allocate(iloc(2,10000))
    allocate(val(10000))
    nsmall=floor(real(ntot)/real(nmpi-1))
    nextra=ntot-nsmall*(nmpi-1)
    ibeg=nsmall*(impi-1)
    if(impi<nextra) then
       ibeg=ibeg+impi
    end if
    iend=iend+nsmall
    if(impi <= nsmall) then
      iend=iend+1
    end if
    if(impi/=0) then
      local_i="junki"
      call parcat(local_i,impi)
      local_v="junkv"
      call parcat(local_v,impi)
      call auxinout(local_i)
      call auxinout(local_v)
    end if
    do while(ndone < ntot) 
       nblock=min(10000,ntot-ndone)
       ndone=ndone+nblock
       nsmall=0
       if(impi==0) then
         call sreed("intag",buf,nblock*12)
       end if
       if(impi/=0)  then
         call MPI_RECV( buf, 3*nblock, MPI_REAL, impi-1,23, MPI_COMM_WORLD,status, ierr ) 
       end if
       if(impi/= nmpi-1) then
         call MPI_SEND( buf, 3*nblock, MPI_REAL, impi+1,23, MPI_COMM_WORLD, ierr ) 
       end if
       if(impi/=0) then
         do i=1,nblock
           if(nint(buf(1,i))>=ibeg .and. nint(buf(1,i)) < iend) then
             nsmall=nsmall+1 
             iloc(1:2,nsmall)= nint(buf(1:2,i))
             val(nsmall)=buf(3,i)
           end if
         end do
         if(nsmall>0) then
            call srite(local_i,iloc,nsmall*8)
            call srite(local_v,val,nsmall*4)
         end if
         nlocal=nlocal+nsmall
       end if
    end do
    deallocate(iloc,val)
  end subroutine

  subroutine do_diag(total_sparse,in_vec,out_vec)
    integer :: counter,total_sparse
    integer status(MPI_STATUS_SIZE) ,ierr
    integer, pointer :: iloc(:,:)
    real :: in_vec(:),out_vec(:);
    real, pointer    :: val(:) 
    real, pointer :: temp_vec(:)
    integer :: total_sparse_read,isend,max_sparse,it

   if(first) then
     call distribute_mat(total_sparse)
     first=.false.
   end if

   allocate(temp_vec(size(in_vec)))

   call sseek(local_v,0,0)
   call sseek(local_i,0,0)
   total_sparse_read=0

  call MPI_RECV( in_vec, size(in_vec), MPI_REAL, impi-1,25, MPI_COMM_WORLD,status, ierr ) 
   if(impi/= nmpi-1) then
     call MPI_SEND( out_vec, size(in_vec), MPI_REAL, impi+1,25, MPI_COMM_WORLD, ierr ) 
    end if

   allocate(iloc(2,50000),val(50000))
   counter=1
   out_vec=0.
   read: do while (total_sparse_read<nlocal)
     ! READ INPUT DATA AGAIN
     
     max_sparse=min(nlocal-total_sparse_read,50000)
     call sreed(local_v,val,size(val)*4)
     call sreed(local_i,iloc,size(iloc)*4)

     total_sparse_read=max_sparse+total_sparse_read

     ! 1)COMPUTE OFF DIAGONAL
     do it=1,max_sparse
        if (iloc(1,it)/=iloc(2,it)) then
           out_vec(iloc(1,it))=out_vec(iloc(1,it))+&
             in_vec(iloc(2,it))*val(it)
           out_vec(iloc(2,it))=out_vec(iloc(2,it))+&
             in_vec(iloc(1,it))*val(it)
        end if
     end do
     counter=counter+1 
   end do read
   if(impi/=1)  then
     call MPI_RECV(temp_vec,size(temp_vec), MPI_REAL,impi-1,24,MPI_COMM_WORLD,status, ierr ) 
    do it=1,size(temp_vec)
       out_vec(it)=out_vec(it)+temp_vec(it)
   end do
   isend=impi+1
   if(impi==nmpi-1) isend=0
     call MPI_SEND(out_vec, size(out_vec), MPI_REAL, isend,24, MPI_COMM_WORLD, ierr ) 
   end if
   deallocate(temp_vec,iloc,val)
 end subroutine

  subroutine sparse_mat_mult3(in_vec,out_vec,mem_alloc,d_diag,d_vec_exp,non_zero_mask,total_sparse)
    integer               ::non_zero_mask,it,max_sparse,counter
    integer               ::total_sparse_read,mem_alloc,total_sparse
    real, dimension(:,:),allocatable  ::sparse_mat
    integer, dimension(:,:),allocatable  ::iloc
    real, dimension(:),allocatable  ::val
    real,dimension(:)     ::in_vec,out_vec,d_diag,d_vec_exp
    integer :: nm,nbuf,n,isend
    integer status(MPI_STATUS_SIZE) ,ierr
    real, pointer :: temp_vec(:)

    ! GO TO BEGINING OF SPARSE MATRIX FILE


   if(first) then
     call distribute_mat(total_sparse)
     first=.false.
   end if
   allocate(temp_vec(size(in_vec)))

   call MPI_SEND( out_vec, size(in_vec), MPI_REAL, impi+1,25, MPI_COMM_WORLD, ierr ) 
   do it=1,non_zero_mask
      out_vec(it)=in_vec(it)*d_diag(it)
   end do
   call MPI_RECV(temp_vec,size(temp_vec), MPI_REAL,nmpi-1,24,MPI_COMM_WORLD,status, ierr ) 
   do it=1,non_zero_mask
      out_vec(it)=out_vec(it)+temp_vec(it)
   end do
  deallocate(temp_vec)
  end subroutine sparse_mat_mult3

end module paint_mod3d_mpi
