#!/usr/bin/env python
import SEP.args
import SEP.pj_base
import SEP.pf_copy
import commands
import re
import SEP.pf_split
import SEP.sepfile
import SEP.util


SEP.util.debug=1   #Tell me where I have an error
SEP.spawn.print_all=1  #Print every command


class  segment_par(SEP.pj_base.par_job):
  def __init__(self,args,name="Segment"):
    """Initialize the Fdmod parallel job"""
    self.args=args
    SEP.pj_base.par_job.__init__(self,name)

  def prep_run(self):
    pfiles={}
    njobs=self.args.param("njobs")
    pfiles['input']=SEP.pf_copy.parfile(name=self.args.param("intag"),
     tag="<", usage="INPUT",njobs=njobs)
    inS=SEP.sepfile.sep_file(file=self.args.param("intag"))
    self.args.del_par("input")
    nsz=inS.axis(1)[0]*inS.axis(2)[0]*inS.axis(3)[0]
    

    pfiles['output']=SEP.pf_split.parfile(name=self.args.param("outtag"),
     tag=">", usage="OUTPUT",njobs=njobs,dff_axis=2,nblock=njobs)
    self.args.del_par("output")

    if self.args.param("mask_in"):
      pfiles['mask_in']=SEP.pf_copy.parfile(name=self.args.param("mask_in"),
       tag="mask_in=", usage="INPUT",njobs=njobs)
      maskS=SEP.sepfile.sep_file(file=self.args.param("mask_in"))
      print "2BB Add  add=-1 %s|Attr "%self.args.param("mask_in")
      stat,out=commands.getstatusoutput("Add  add=-1 %s>x.H"%self.args.param("mask_in"))
      print "1BB",stat,out
      stat,out=commands.getstatusoutput("<x.H Attr ")
      find=re.compile("number of nonzero samples\s+=\s+(\d+)").search(out)
      if not find: SEP.util.err("Trouble parsing Attr %s "%out)
      nrow=nsz-int(find.group(1))
      self.args.del_par("mask_in")
    else:
      self.nrow=inS.size() 

    if self.args.param("upper_bnd_vec"):
      pfiles['upper_bnd_vec']=SEP.pf_copy.parfile(name=self.args.param("upper_bnd_vec"),
       tag="upper_bnd_vec=", usage="INPUT",njobs=njobs)
      self.args.del_par("upper_bnd_vec")

    if self.args.param("lower_bnd_vec"):
      pfiles['lower_bnd_vec']=SEP.pf_copy.parfile(name=self.args.param("lower_bnd_vec"),
       tag="lower_bnd_vec=", usage="INPUT",njobs=njobs)
      self.args.del_par("lower_bnd_vec")

    if self.args.param("lower_bnd"):
      pfiles['lower_bnd']=SEP.pf_copy.parfile(name=self.args.param("lower_bnd"),
       tag="lower_bnd=", usage="INPUT",njobs=njobs)
      self.args.del_par("lower_bnd")

    if self.args.param("upper_bnd"):
      pfiles['upper_bnd']=SEP.pf_copy.parfile(name=self.args.param("upper_bnd"),
       tag="upper_bnd=", usage="INPUT",njobs=njobs)
      self.args.del_par("upper_bnd")

    if self.args.param("first_attribute"):
      pfiles['first_attribute']=SEP.pf_copy.parfile(name=self.args.param("first_attribute"),
       tag="first_attribute=", usage="INPUT",njobs=njobs)
      self.args.del_par("first_attribute")

    self.add_param("program",self.args.param("program"))

    self.add_pars(SEP.pj_func._cl_par_params())
    self.add_param("files",pfiles)
    self.add_param("sect_pars",self.sect_pars(njobs,nrow))
    self.add_param("global_pars",self.args)
    SEP.pj_base.par_job.prep_run(self,None)



  def  sect_pars(self,nblock,nrow):

    sect_pars={}
    beg=0
    nsmall=int(float(nrow)/float(nblock))
    nextra=int(nrow)-int(nsmall)*int(nblock)
    for i in range(int(nblock)):
      sect_pars[str(i)]=SEP.args.basic(name=str(i))
      ntot=nsmall
      if i < nextra:   ntot=ntot+1
      sect_pars[str(i)].add_param("first_index",beg+1)
      sect_pars[str(i)].add_param("last_index",beg+ntot)
      beg=beg+ntot
    return sect_pars




args=SEP.args.sep_args()
op=segment_par(args)
op.prep_run()
