program Salt_mask  
  ! Create a mask along a saltboundary  from a 3D velocity cube.   Jesse Lomask 9/21/04

  use sep
  use lint2
  use warp_mod
  use sorting_mod
  use paint_mod3d

  implicit none

  integer				::nt, nx, ny, non_zero_mask,istatus,bnd_index,mask_shift
  integer				::it,ix,iy,ir,iq,iz,ip,it_shft,ix_shft,iy_shft
  real::o1,o2,o3,d1,d2,d3,maxdat,mindat,rangedat

  real, pointer, dimension(:,:,:)	::data, norm_d,mask_out,inner_bnd,outer_bnd,simple_mask
!!$  real, dimension(:,:,:)                ::box_grid,box_grid_colx,box_grid_coly,box_grid_row,box_grid_bnds


  ! HOOK UP SEP PARAMETER INPUT FACILITY
  call sep_init(SOURCE)

!!$  ! INPUT PARAMETERS
  call from_history("n1",nt); call from_history("n2",nx);
  call from_history("n3",ny);
  call from_history("o1",o1); call from_history("o2",o2); call from_history("o3",o3);
  call from_history("d1",d1); call from_history("d2",d2); call from_history("d3",d3);


  ! ALLOCATE INPUT DATA AND MASK
  allocate(data(nt,nx,ny),mask_out(nt,nx,ny),norm_d(nt,nx,ny),outer_bnd(nt,nx,ny),inner_bnd(nt,nx,ny),simple_mask(nt,nx,ny))

  ! READ INPUT DATA
  call sep_read(data)

  ! INVERT AND NORMALIZE THE DATA

  maxdat=maxval(data)
  mindat=minval(data)
  rangedat=maxdat-mindat
  norm_d=(data/rangedat-mindat/rangedat)

  deallocate(data)


  write(0,*) 'initialize mask'
  mask_out=0.
  mask_shift=10
  do iy=1,ny
     do ix=1,nx
        do it=1,nt
           if (norm_d(it,ix,iy)>.99) then
              mask_out(it,ix,iy)=.25
           else
              mask_out(it,ix,iy)=.75           
           end if
        end do
     end do
     write(0,*) 'initializing mask iy',iy,'out of',ny
  end do
  write(0,*) 'making mask'
  do iy=2,ny
     do ix=2,nx
        do it=2,nt
           if (norm_d(it,ix,iy)>.99.and.(norm_d(it-1,ix,iy)<.99.or.norm_d(it,ix-1,iy)<.99.or.norm_d(it,ix,iy-1)<.99)) then
              mask_out(max(it-5,1):min(it+mask_shift,nt),max(ix-5,1):min(ix+mask_shift,nx), max(iy-5,1):min(iy+mask_shift,ny))=1.            
           end if


        end do
     end do
     write(0,*) 'making mask iy',iy,'out of',ny
  end do

  do iy=1,ny
     do ix=1,nx
        do it=1,nt
           if (norm_d(it,ix,iy)>.99) then
              simple_mask(it,ix,iy)=1.
           else
              simple_mask(it,ix,iy)=-.1
           end if
        end do
     end do
  end do



  write(0,*) 'extracting boundaries'
  outer_bnd=0.
  inner_bnd=0.
  bnd_index=0
  do iy=1,ny
     do ix=1,nx
        do it=1,nt
           if (mask_out(it,ix,iy)==1) then
              do ip=1,27
                 it_shft=mod(ip-1,3)-1
                 ix_shft=(ip-1)/3-1-3*((ip-1)/9)
                 iy_shft=(ip-1)/9-1

                 if (mask_out(max(1,min(nt,it+it_shft)),max(1,min(nx,ix+ix_shft)),max(1,min(ny,iy+iy_shft)))==.25) then
                    inner_bnd(it,ix,iy)=1.
                 end if

              end do
!!$           end if
!!$           if (mask_out(it,ix,iy)==1) then
              do ip=1,27
                 it_shft=mod(ip-1,3)-1
                 ix_shft=(ip-1)/3-1-3*((ip-1)/9)
                 iy_shft=(ip-1)/9-1

                 if (mask_out(max(1,min(nt,it+it_shft)),max(1,min(nx,ix+ix_shft)),max(1,min(ny,iy+iy_shft)))==.75) then
                    bnd_index=bnd_index+1
                    outer_bnd(it,ix,iy)=1.
                 end if

              end do
           end if


        end do
     end do
     write(0,*) 'extracting bounds iy',iy,'out of',ny
  end do
!!$
!!$


  call to_history("n1",nt); call to_history("n2",nx) 
  call to_history("n3",ny)
  call to_history("d1",d1); call to_history("d2",d2)
  call to_history("d3",d3)
  call to_history("o1",o1); call to_history("o2",o2)
  call to_history("o3",o3)
  call to_history("esize",4)
  call sep_write(mask_out)
  deallocate(mask_out)


  call to_history("n1",nt,"inner_bnd"); call to_history("n2",nx,"inner_bnd") 
  call to_history("n3",ny,"inner_bnd")
  call to_history("d1",d1,"inner_bnd"); call to_history("d2",d2,"inner_bnd")
  call to_history("d3",d3,"inner_bnd")
  call to_history("o1",o1,"inner_bnd"); call to_history("o2",o2,"inner_bnd")
  call to_history("o3",o3,"inner_bnd")
  call to_history("esize",4,"inner_bnd")
  call sep_write(inner_bnd,"inner_bnd")
  deallocate(inner_bnd)



  call to_history("n1",nt,"outer_bnd"); call to_history("n2",nx,"outer_bnd") 
  call to_history("n3",ny,"outer_bnd")
  call to_history("d1",d1,"outer_bnd"); call to_history("d2",d2,"outer_bnd")
  call to_history("d3",d3,"outer_bnd")
  call to_history("o1",o1,"outer_bnd"); call to_history("o2",o2,"outer_bnd")
  call to_history("o3",o3,"outer_bnd")
  call to_history("esize",4,"outer_bnd")
  call sep_write(outer_bnd,"outer_bnd")
  deallocate(outer_bnd)


  call to_history("n1",nt,"simple_mask"); call to_history("n2",nx,"simple_mask") 
  call to_history("n3",ny,"simple_mask")
  call to_history("d1",d1,"simple_mask"); call to_history("d2",d2,"simple_mask")
  call to_history("d3",d3,"simple_mask")
  call to_history("o1",o1,"simple_mask"); call to_history("o2",o2,"simple_mask")
  call to_history("o3",o3,"simple_mask")
  call to_history("esize",4,"simple_mask")
  call sep_write(simple_mask,"simple_mask")
  deallocate(simple_mask)


end program Salt_mask

